#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_sequence.interfaces import ISequentialIdInfo, ISequentialIdTarget, ISequentialIntIds

# import packages
from persistent import Persistent
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.registry import get_utility
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(ISequentialIdInfo)
class SequentialIdInfo(Persistent):
    """Sequential ID info"""

    oid = FieldProperty(ISequentialIdInfo['oid'])
    hex_oid = FieldProperty(ISequentialIdInfo['hex_oid'])

    def get_full_oid(self):
        sequence = get_utility(ISequentialIntIds)
        return sequence.get_full_oid(self.oid)

    def get_short_oid(self):
        sequence = get_utility(ISequentialIntIds)
        return sequence.get_short_oid(self.oid)

    def get_base_oid(self):
        sequence = get_utility(ISequentialIntIds)
        return sequence.get_base_oid(self.oid)

    @property
    def public_oid(self):
        return self.get_short_oid()


SEQUENCE_INFO_KEY = 'pyams_sequence.info'


@adapter_config(context=ISequentialIdTarget, provides=ISequentialIdInfo)
def sequential_id_info_factory(context):
    """Sequential ID info factory"""
    return get_annotation_adapter(context, SEQUENCE_INFO_KEY, SequentialIdInfo,
                                  notify=False, locate=False)
